@@ -1,18 +1,42 @@
 """Definition of the identity predicates."""
 
-import socket
-import struct
-import threading
+
+# declare what should be exported
+__all__ = [
+    'All',
+    'Any',
+    'NotAny',
+    'CompoundPredicate',
+    'IdentityPredicateHelper',
+    'Predicate',
+    'SecureObject',
+    'SecureResource',
+    'from_host',
+    'from_any_host',
+    'in_all_groups',
+    'in_any_group',
+    'in_group',
+    'has_all_permissions',
+    'has_any_permission',
+    'has_permission',
+    'not_anonymous',
+    'require',
+]
+
+
 import types
-from turbogears import config
+
 from cherrypy import request
+from turbogears import config
 from turbogears.identity.exceptions import *
-from turbogears.identity import *
+from turbogears.identity.base import current
 from turbogears.decorator import weak_signature_decorator
+from turbogears.util import match_ip
 
 
 class Predicate(object):
     """Generic base class for testing true or false for a condition."""
+
     def eval_with_object(self, obj, errors=None):
         """Determine whether predicate is True or False for the given object."""
         raise NotImplementedError
@@ -25,37 +49,41 @@ class Predicate(object):
 
 class CompoundPredicate(Predicate):
     """A predicate composed of other predicates."""
+
     def __init__(self, *predicates):
         self.predicates = predicates
 
 
 class All(CompoundPredicate):
-    """Logical and of all sub-predicates.
+    """Logical 'and' of all sub-predicates.
 
     This compound predicate evaluates to true only if all of its sub-predicates
     evaluate to true for the given input.
 
     """
+    error_message = "One of the predicates denied access"
+
     def eval_with_object(self, obj, errors=None):
-        """Return true if all sub-predicates evaluate to true."""
+        """Return true if all sub-predicates evaluate to true.
+        """
         for p in self.predicates:
             if not p.eval_with_object(obj, errors):
+                self.append_error_message(errors)
                 return False
         return True
 
 
 class Any(CompoundPredicate):
-    """Logical or of all sub-predicates.
+    """Logical 'or' of all sub-predicates.
 
     This compound predicate evaluates to true if any one of its sub-predicates
     evaluates to true for the given input.
 
     """
-    error_message = "No predicates were able to grant access"
+    error_message = "No predicate was able to grant access"
 
     def eval_with_object(self, obj, errors=None):
-        """Return true if any sub-predicate evaluates to true.
-        """
+        """Return true if any sub-predicate evaluates to true."""
         for p in self.predicates:
             if p.eval_with_object(obj, None):
                 return True
@@ -63,8 +91,27 @@ class Any(CompoundPredicate):
         return False
 
 
+class NotAny(CompoundPredicate):
+    """Locigal 'nor' of all sub-predicates.
+
+    A compound predicate that evaluates to true only if no sub-predicates
+    evaluate to true for the given input.
+
+    """
+    error_message= "One of the predicates did not deny access"
+
+    def eval_with_object(self, obj, errors=None):
+        """Return true if no sub-predicates evaluate to true."""
+        for p in self.predicates:
+            if p.eval_with_object(obj, errors):
+                self.append_error_message(errors)
+                return False
+        return True
+
+
 class IdentityPredicateHelper(object):
     """A mix-in helper class for Identity Predicates."""
+
     def __nonzero__(self):
         return self.eval_with_object(current)
 
@@ -85,9 +132,10 @@ class in_group(Predicate, IdentityPredicateHelper):
 
 class in_all_groups(All, IdentityPredicateHelper):
     """Predicate for requiring membership in a number of groups."""
+
     def __init__(self, *groups):
         group_predicates = [in_group(g) for g in groups]
-        super(in_all_groups,self).__init__(*group_predicates)
+        super(in_all_groups, self).__init__(*group_predicates)
 
 
 class in_any_group(Any, IdentityPredicateHelper):
@@ -97,7 +145,7 @@ class in_any_group(Any, IdentityPredicateHelper):
     def __init__(self, *groups):
         self.group_list = ", ".join(groups)
         group_predicates = [in_group(g) for g in groups]
-        super(in_any_group,self).__init__(*group_predicates)
+        super(in_any_group, self).__init__(*group_predicates)
 
 
 class not_anonymous(Predicate, IdentityPredicateHelper):
@@ -128,9 +176,10 @@ class has_permission(Predicate, IdentityPredicateHelper):
 
 class has_all_permissions(All, IdentityPredicateHelper):
     """Predicate for checking whether the visitor has all permissions."""
+
     def __init__(self, *permissions):
         permission_predicates = [has_permission(p) for p in permissions]
-        super(has_all_permissions,self).__init__(*permission_predicates)
+        super(has_all_permissions, self).__init__(*permission_predicates)
 
 
 class has_any_permission(Any, IdentityPredicateHelper):
@@ -140,25 +189,15 @@ class has_any_permission(Any, IdentityPredicateHelper):
     def __init__(self, *permissions):
         self.permission_list = ', '.join(permissions)
         permission_predicates = [has_permission(p) for p in permissions]
-        super(has_any_permission,self).__init__(*permission_predicates)
+        super(has_any_permission, self).__init__(*permission_predicates)
 
 
 def _remoteHost():
-    return request.headers.get('X-Forwarded-For', request.headers.get(
-        'Remote-Addr', '')).split(',')[-1].strip()
-
-
-def _match_ip(cidr, ip):
-    if not '/' in cidr:
-        return cidr == ip
     try:
-        b, m = cidr.split('/')
-        shift = 32 - int(m)
-        a1 = struct.unpack('!L', socket.inet_aton(b))[0] >> shift
-        a2 = struct.unpack('!L', socket.inet_aton(ip))[0] >> shift
-        return a1 == a2
-    except Exception:
-        return False
+        return request.headers.get('X-Forwarded-For', request.headers.get(
+            'Remote-Addr', '')).rsplit(',', 1)[-1].strip() or None
+    except AttributeError:
+        raise RequestRequiredException()
 
 
 class from_host(Predicate, IdentityPredicateHelper):
@@ -176,7 +215,7 @@ class from_host(Predicate, IdentityPredicateHelper):
     def eval_with_object(self, obj, errors=None):
         """Match the visitor's host against the criteria."""
         ip = _remoteHost()
-        if _match_ip(self.host, ip):
+        if ip and match_ip(self.host, ip):
             return True
         self.append_error_message(errors)
         return False
@@ -198,6 +237,7 @@ def require(predicate, obj=None):
     of the groups specified and has the permissions required.
 
     """
+
     def entangle(fn):
         def require(func, self, *args, **kwargs):
             try:
@@ -215,6 +255,7 @@ def require(predicate, obj=None):
 
 def _secureResourceDecorator(fn):
     """A decorator function used by the SecureResourceMeta metaclass."""
+
     def _wrapper(self, *args, **kwargs):
         predicate = getattr(self.__class__, 'require', None)
         try:
@@ -250,9 +291,9 @@ class SecureResource(object):
     def __getattribute__(self, name):
         from turbogears import controllers
         if name[:3] == '_cp' or name == 'require':
-            return object.__getattribute__(self,name)
+            return object.__getattribute__(self, name)
         try:
-            value = object.__getattribute__(self,name)
+            value = object.__getattribute__(self, name)
             try:
                 predicate = object.__getattribute__(self, 'require')
             except AttributeError:
@@ -270,7 +311,7 @@ class SecureResource(object):
             # Some other property
             return value
         except IdentityException, e:
-            errors= [str(e)]
+            errors = [str(e)]
         raise IdentityFailure(errors)
 
 
@@ -283,8 +324,8 @@ class SecureObject(object):
 
     def __getattribute__(self, name):
         from turbogears import controllers
-        if name[:3] == '_cp' or name in ('_object', '_require', '_exclude'):
-            return object.__getattribute__(self,name)
+        if name.startswith('_cp') or name in ('_object', '_require', '_exclude'):
+            return object.__getattribute__(self, name)
         try:
             obj = object.__getattribute__(self, '_object')
             value = getattr(obj, name)
