@@ -1,29 +1,50 @@
-"Things to do when TurboGears is imported."
+"""Things to do when the TurboGears server is started."""
 
-import os
+__all__ = [
+    'call_on_startup',
+    'call_on_shutdown',
+    'reloader_thread',
+    'start_bonjour',
+    'stop_bonjour',
+    'start_server',
+    'startTurboGears',
+    'stopTurboGears',
+    'webpath',
+]
+
+import atexit
 import errno
 import logging
+import os
 import sys
-import time
-import atexit
 import signal
+import time
+
+from os.path import abspath, exists
 
 import pkg_resources
 import cherrypy
 from cherrypy import _cputil, request, server
-from formencode.variabledecode import NestedVariables
 from cherrypy._cpwsgi import wsgiApp, CPHTTPRequest
 from cherrypy._cpwsgiserver import CherryPyWSGIServer
 
-from turbogears import config, scheduler, database
-from turbogears import view
-from turbogears.database import hub_registry, EndTransactionsFilter
-
-log = logging.getLogger("turbogears.startup")
-
 pkg_resources.require("TurboGears")
 
+from turbogears import config, database, scheduler, view
+from turbogears.database import hub_registry, EndTransactionsFilter
+from turbogears.filters import (MonkeyDecodingFilter, NestedVariablesFilter,
+    SafeMultipartFilter, VirtualPathFilter)
 
+
+# module globals
+DNS_SD_PID = None
+call_on_startup = []
+call_on_shutdown = []
+webpath = ''
+log = logging.getLogger("turbogears.startup")
+
+
+# module private functions
 def reloader_thread(freq):
     """Monkeypatch for the reloader provided by CherryPy.
 
@@ -40,10 +61,10 @@ def reloader_thread(freq):
     mtimes = {}
     package = config.get("autoreload.package", None)
     if package is None:
-        print \
-"""TurboGears requires autoreload.package to be set. It can be an empty
-value, which will use CherryPy's default behavior which is to check
-every module. Setting an actual package makes the check much faster."""
+        print ("TurboGears requires autoreload.package to be set. "
+            "It can be an empty value, which will use CherryPy's default "
+            "behavior which is to check every module. Setting an actual "
+            "package makes the check much faster.")
         return
     while cherrypy.lib.autoreload.RUN_RELOADER:
         if package:
@@ -78,12 +99,28 @@ every module. Setting an actual package makes the check much faster."""
 
 cherrypy.lib.autoreload.reloader_thread = reloader_thread
 
-webpath = ''
+old_object_trail = _cputil.get_object_trail
 
-DNS_SD_PID = None
+# hang on to object trail to use it to find an app root if need be
+def get_object_trail(object_path=None):
+    trail = old_object_trail(object_path)
+    try:
+        request.object_trail = trail
+    except AttributeError:
+        pass
+    return trail
+
+_cputil.get_object_trail = get_object_trail
 
 
+# module public functions
 def start_bonjour(package=None):
+    """Register the TurboGears server with the Bonjour framework.
+
+    Currently only Unix-like systems are supported where either the 'avahi'
+    daemon (Linux etc.) is available or the 'dns-sd' program (Mac OS X).
+
+    """
     global DNS_SD_PID
     if DNS_SD_PID:
         return
@@ -107,13 +144,13 @@ def start_bonjour(package=None):
         # avahi isn't the one running, then this won't work.  We should either
         # try registering with both or checking what service is running and use
         # that.  Program availability on the filesystem was never enough...
-        if os.path.exists(cmd):
+        if exists(cmd):
             DNS_SD_PID = os.spawnv(os.P_NOWAIT, cmd, [cmd]+args)
             atexit.register(stop_bonjour)
             break
 
-
 def stop_bonjour():
+    """Stop the bonjour publishing daemon if it is running."""
     if not DNS_SD_PID:
         return
     try:
@@ -121,70 +158,38 @@ def stop_bonjour():
     except OSError:
         pass
 
-
-class VirtualPathFilter(object):
-    """Filter that makes CherryPy ignorant of a URL root path.
-
-    That is, you can mount your app so the URI "/users/~rdel/myapp/"
-    maps to the root object "/".
-
-    """
-
-    def __init__(self, webpath=''):
-        webpath = webpath.rstrip('/')
-        if webpath and not webpath.startswith('/'):
-             webpath = '/' + webpath
-        self.webpath = webpath
-
-    def before_request_body(self):
-        """Determine the relevant path info by stripping of prefixes.
-
-        Strips webpath and SCRIPT_NAME from request.object_path and
-        sets request.path_info (since CherryPy 2 does not set it).
-
-        """
-        webpath = self.webpath
-        try:
-            webpath += request.wsgi_environ['SCRIPT_NAME'].rstrip('/')
-        except (AttributeError, KeyError):
-            pass
-        if webpath:
-            if request.object_path.startswith(webpath):
-                request.object_path = request.object_path[len(webpath):] or '/'
-            if request.path.startswith(webpath):
-                request.path_info = request.path[len(webpath):] or '/'
-            else:
-                request.path_info = request.path
-                # check for webpath only if not forwarded
-                try:
-                    if not request.wsgi_environ['HTTP_X_FORWARDED_SERVER']:
-                        raise KeyError
-                except (AttributeError, KeyError):
-                    raise cherrypy.NotFound(request.path)
-        else:
-            request.path_info = request.path
-
-
-class NestedVariablesFilter(object):
-
-    def before_main(self):
-        if hasattr(request, 'params'):
-            request.params = NestedVariables.to_python(request.params or {})
-
-
 def startTurboGears():
     """Handles TurboGears tasks when the CherryPy server starts.
 
-    This adds the "tg_js" configuration to make MochiKit accessible.
-    It also turns on stdlib logging when in development mode.
+    This performs the following initialization tasks (in given order):
+
+    * Turns off CherryPy's logging filter when in development mode
+    * If logging is not already set up, turns on old-style stdlib logging.
+    * Adds a static filter for TurboGears's static files (URL '/tg_static').
+    * Adds a static filter for TurboGears's JavaScript files (URL '/tg_js').
+    * Loads the template engines and the base templates.
+    * Adds the CherryPy request filters to the root controller.
+    * Adds the decoding filter to the root URL ('/') if enabled in the
+      configuration.
+    * Registers the server with the Bonjour framework, if available.
+    * Calls 'turbogears.database.bind_metadata' when using SQLAlchemy.
+    * Loads all turbogears.extensions entry points and calls their
+      'start_extension' method.
+    * Calls the callables registered in 'turbogears.call_on_startup'.
+    * Starts the TurboGears scheduler.
 
     """
-    # Set up old-style logging
+    global webpath
+    conf = config.get
+    rfn = pkg_resources.resource_filename
+
     cherrypy.config.environments['development'][
         'log_debug_info_filter.on'] = False
 
-    if not config.get('tg.new_style_logging'):
-        if config.get('server.log_to_screen'):
+    # XXX: obsolete --> to be removed
+    # Set up old-style logging
+    if not conf('tg.new_style_logging'):
+        if conf('server.log_to_screen'):
             setuplog = logging.getLogger()
             setuplog.setLevel(logging.DEBUG)
             fmt = logging.Formatter(
@@ -194,7 +199,7 @@ def startTurboGears():
             handler.setFormatter(fmt)
             setuplog.addHandler(handler)
 
-        logfile = config.get('server.log_file')
+        logfile = conf('server.log_file')
         if logfile:
             setuplog = logging.getLogger('turbogears.access')
             setuplog.propagate = 0
@@ -205,40 +210,21 @@ def startTurboGears():
             setuplog.addHandler(handler)
 
     # Add static filters
-    resource_filename = pkg_resources.resource_filename
     config.update({'/tg_static': {
-            'static_filter.on': True,
-            'static_filter.dir':
-                os.path.abspath(resource_filename(__name__, 'static')),
-            'log_debug_info_filter.on': False,
-        }})
+        'static_filter.on': True,
+        'static_filter.dir': abspath(rfn(__name__, 'static')),
+        'log_debug_info_filter.on': False,
+    }})
     config.update({'/tg_js': {
-            'static_filter.on': True,
-            'static_filter.dir':
-                os.path.abspath(resource_filename(__name__, 'static/js')),
-            'log_debug_info_filter.on': False,
-        }})
-    mochikit_version = config.get(
-        'tg.mochikit_version', config.get('tg_mochikit.version', '1.3'))
-    mochikit_suffix = '_'.join(mochikit_version.split('.', 2)[:2])
-    mochikit_file = os.path.abspath(resource_filename(
-        __name__, 'static/js/MochiKit_%s.js' % mochikit_suffix))
-    if os.path.exists(mochikit_file):
-        config.update({'/tg_js/MochiKit.js': {
-            'static_filter.on': True,
-            'static_filter.file': mochikit_file,
-        }})
-        config.update({'/tg_widgets/turbogears/js/MochiKit.js': {
-            'static_filter.on': True,
-            'static_filter.file': mochikit_file
-        }})
-    else:
-        log.warning("MochiKit version %s not available" % mochikit_version)
+        'static_filter.on': True,
+        'static_filter.dir': abspath(rfn(__name__, 'static/js')),
+        'log_debug_info_filter.on': False,
+    }})
     # Add decoding filter
-    if config.get('decoding_filter.on', path='/') is None:
+    if conf('decoding_filter.on', path='/') is None:
         config.update({'/': {
             'decoding_filter.on': True,
-            'decoding_filter.encoding': config.get('kid.encoding', 'utf8')
+            'decoding_filter.encoding': conf('kid.encoding', 'utf8')
         }})
 
     # Initialize template engines and load base templates
@@ -246,46 +232,90 @@ def startTurboGears():
     view.loadBaseTemplates()
 
     # Add request filters
-    global webpath
-    webpath = config.get('server.webpath') or ''
+    webpath = conf('server.webpath') or ''
 
     if getattr(cherrypy, 'root', None):
         if not hasattr(cherrypy.root, '_cp_filters'):
             cherrypy.root._cp_filters = []
-        cherrypy.root._cp_filters.extend([VirtualPathFilter(webpath),
-            EndTransactionsFilter(), NestedVariablesFilter()])
+        extra_filters = [
+            VirtualPathFilter(webpath),
+            EndTransactionsFilter(),
+            NestedVariablesFilter(),
+            SafeMultipartFilter()
+        ]
+        # Do not add filters twice which are already present
+        for cp_filter in cherrypy.root._cp_filters[:]:
+            for candidate in extra_filters:
+                if candidate.__class__ == cp_filter.__class__:
+                    extra_filters.remove(candidate)
+                    break
+        cherrypy.root._cp_filters.extend(extra_filters)
+
+    # Monkey patch CherryPy Decoding filter: injects our replacement filter
+    # MonkeyDecodingFilter into the CherryPy filter chain
+    decoding_filter = MonkeyDecodingFilter()
+    for index, active_filter in enumerate(
+            cherrypy.filters._filterhooks.get('before_main', [])):
+        if active_filter.im_class == \
+                cherrypy.filters.decodingfilter.DecodingFilter:
+            cherrypy.filters._filterhooks['before_main'].pop(index)
+            if conf('decoding_filter.on', False, path='/'):
+                log.info("Request decoding filter activated.")
+                cherrypy.filters._filterhooks['before_main'].insert(
+                        index, decoding_filter.before_main)
+
 
     webpath = webpath.lstrip('/')
     if webpath and not webpath.endswith('/'):
         webpath += '/'
 
     # Register server with Bonjour framework
-    bonjoursetting = config.get('tg.bonjour', None)
-    if bonjoursetting or config.get('server.environment') == 'development':
+    bonjoursetting = conf('tg.bonjour', None)
+    if bonjoursetting or conf('server.environment') == 'development':
         start_bonjour(bonjoursetting)
 
     # Bind metadata for SQLAlchemy
-    if config.get("sqlalchemy.dburi"):
-        database.get_engine()
+    if conf('sqlalchemy.dburi'):
+        database.bind_metadata()
 
     # Start all TurboGears extensions
     extensions = pkg_resources.iter_entry_points('turbogears.extensions')
     for entrypoint in extensions:
-        ext = entrypoint.load()
+        # We try to load the extension and run its 'start_extension'
+        # method ,if present. If either fails, we simply log the exception and
+        # continue, because a) when the autoreloader is active, unhandled
+        # exceptions in the startup phase will not stop the server and
+        # b) faulty extensions (which may be from a different package)
+        # should not crash the server.
+        try:
+            ext = entrypoint.load()
+        except Exception, e:
+            log.exception("Error loading TurboGears extension plugin '%s': %s",
+                entrypoint, e)
+            continue
         if hasattr(ext, 'start_extension'):
-            ext.start_extension()
+            try:
+                ext.start_extension()
+            except Exception, e:
+                log.exception("Error starting TurboGears extension '%s': %s",
+                    entrypoint, e)
 
     # Call registered startup functions
     for item in call_on_startup:
         item()
 
     # Start the scheduler
-    if config.get('tg.scheduler', False):
+    if conf('tg.scheduler', False):
         scheduler._start_scheduler()
-        log.info('Scheduler started')
-
+        log.info("Scheduler started")
 
 def stopTurboGears():
+    """Handles TurboGears tasks when the CherryPy server stops.
+
+    Ends all open database transactions, shuts down all extensions, calls user
+    provided shutdown functions and stops the scheduler.
+
+    """
     # end all transactions and clear out the hubs to
     # help ensure proper reloading in autoreload situations
     for hub in hub_registry:
@@ -297,9 +327,19 @@ def stopTurboGears():
     # Shut down all TurboGears extensions
     extensions= pkg_resources.iter_entry_points( "turbogears.extensions" )
     for entrypoint in extensions:
-        ext= entrypoint.load()
+        try:
+            ext = entrypoint.load()
+        except Exception, e:
+            log.exception("Error loading TurboGears extension plugin '%s': %s",
+                entrypoint, e)
+            continue
         if hasattr(ext, "shutdown_extension"):
-            ext.shutdown_extension()
+            try:
+                ext.shutdown_extension()
+            except Exception, e:
+                log.exception(
+                    "Error shutting down TurboGears extension '%s': %s",
+                    entrypoint, e)
 
     for item in call_on_shutdown:
         item()
@@ -308,21 +348,14 @@ def stopTurboGears():
         scheduler._stop_scheduler()
         log.info("Scheduler stopped")
 
+def start_server(root):
+    cherrypy.root = root
+    if config.get('tg.fancy_exception', False):
+        server.start(server=SimpleWSGIServer())
+    else:
+        server.start()
 
-old_object_trail = _cputil.get_object_trail
-
-# hang on to object trail to use it to find an app root if need be
-def get_object_trail(object_path=None):
-    trail = old_object_trail(object_path)
-    try:
-        request.object_trail = trail
-    except AttributeError:
-        pass
-    return trail
-
-_cputil.get_object_trail = get_object_trail
-
-
+# module classes
 class SimpleWSGIServer(CherryPyWSGIServer):
     """A WSGI server that accepts a WSGI application as a parameter."""
     RequestHandlerClass = CPHTTPRequest
@@ -344,19 +377,8 @@ class SimpleWSGIServer(CherryPyWSGIServer):
             request_queue_size = conf("server.socket_queue_size"))
 
 
-def start_server(root):
-    cherrypy.root = root
-    if config.get('tg.fancy_exception', False):
-        server.start(server=SimpleWSGIServer())
-    else:
-        server.start()
-
-
 if startTurboGears not in server.on_start_server_list:
     server.on_start_server_list.append(startTurboGears)
 
 if stopTurboGears not in server.on_stop_server_list:
     server.on_stop_server_list.append(stopTurboGears)
-
-call_on_startup = []
-call_on_shutdown = []
