@@ -1,37 +1,53 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-from turbogears import widgets, validators, controllers, expose, validate
+from turbogears import widgets, validators, controllers, expose, validate, \
+                       testutil
 from turbogears import config
-from turbogears.testutil import catch_validation_errors, create_request
+from turbogears.testutil import catch_validation_errors, make_app
 import cherrypy
 
-class Request:
-    input_values = {}
-    validation_errors = {}
-
-oldrequest = None
 
 def setup_module():
-    global oldrequest
-    oldrequest = cherrypy.request
-    cherrypy.request = Request()
+    testutil.start_server()
 
 def teardown_module():
-    global oldrequest
-    cherrypy.request = oldrequest
+    testutil.stop_server()
+
 
 def test_rendering():
     """Forms can be rendered"""
-    form = widgets.TableForm(fields=[widgets.TextField("name", label="Your Name")])
-    output = form.render(action="mylocation", format="xhtml")
-    print output
-    assert "Your Name" in output
-    assert 'name="name"' in output
-    assert "submit" in output
+    for Form in (widgets.TableForm, widgets.ListForm):
+        form = Form(
+            fields=[widgets.TextField("name", label="Your Name")])
+        output = form.render(action="mylocation", format="xhtml")
+        assert '<form ' in output and '</form>' in output
+        assert 'action="mylocation"' in output
+        assert 'method="post"' in output
+        assert 'id="form"' in output
+        assert 'name="form"' not in output
+        assert "Your Name" in output
+        assert 'name="name"' in output
+        assert "submit" in output
+        assert (Form is widgets.TableForm) == (
+            '</table>' in output and '</td>' in output)
+        assert (Form is widgets.ListForm) == (
+            '</ul>' in output and '</li>' in output)
+        form = Form(name="myform",
+            fields=[widgets.TextField("myfield")])
+        output = form.render(format="xhtml")
+        assert 'name="myform"' not in output
+        assert 'id="myform"' in output
+        assert 'name="myfield"' in output
+        form = Form(name="myform", use_name=True,
+            fields=[widgets.TextField("myfield")])
+        output = form.render(format="xhtml")
+        assert 'name="myform"' in output
+        assert 'id="myform"' not in output
+        assert 'name="myfield"' in output
 
 def test_input_conversion():
-    "Input for the whole form can be validated and converted"
+    """Input for the whole form can be validated and converted"""
     form = widgets.TableForm(fields=[widgets.TextField("name"),
                       widgets.TextField("age", validator=validators.Int())],
                       submit_text="Submit")
@@ -42,7 +58,7 @@ def test_input_conversion():
     assert not values.has_key("submit")
 
 def test_passing_instance():
-    "You can pass an instance to a form for the value"
+    """You can pass an instance to a form for the value"""
     form = widgets.TableForm(fields=[widgets.TextField("name"),
                       widgets.TextField("age", validator=validators.Int())],
                       submit_text="Submit")
@@ -50,17 +66,15 @@ def test_passing_instance():
         name = "ed"
         age = 892
     output = form.render(Person(), format="xhtml")
-    print output
     assert 'value="ed"' in output
     assert 'value="892"' in output
 
 def test_input_errors():
-    "Data is stored in the request object if there are errors"
+    """Data is stored in the request object if there are errors"""
     form = widgets.TableForm(fields=[widgets.TextField("name"),
                       widgets.TextField("age", validator=validators.Int())])
     values = dict(name="ed", age="ed")
     values, errors = catch_validation_errors(form, values)
-    print errors["age"]
     assert "enter an integer" in str(errors["age"])
 
 class w1(widgets.FormField):
@@ -77,12 +91,12 @@ class w2(widgets.FormField):
     register=False
 
 def test_javascriptsets():
-    "JavaScripts are only added once"
+    """JavaScripts are only added once"""
     form = widgets.TableForm(fields=[w1("foo"), w2("bar")])
     assert len(form.retrieve_javascript()) == 3
 
 def test_csssets():
-    "CSS references are added once"
+    """CSS references are added once"""
     form = widgets.TableForm(fields=[w1("foo"), w2("bar")])
     assert len(form.retrieve_css()) == 1
 
@@ -111,7 +125,6 @@ def test_disabled_widget():
                       widgets.TextField("age", validator=validators.Int())],
                       submit_text="Submit")
     output = form.render(disabled_fields=["age"])
-    print output
     assert "age" not in output
 
 def test_class_attributes_form():
@@ -143,6 +156,13 @@ def test_help_text():
     assert "age here" in output
     assert "name here" in output
 
+def test_values_for_checkbox_list():
+    w = widgets.CheckBoxList(options=list(enumerate(('apples', 'oranges'))))
+    output = w.render()
+    assert 'value="0"' in output and 'value="1"' in output
+    assert 'value=""' not in output and 'value="None"' not in output
+    assert "apples" in output and "oranges" in output
+
 class CallableCounter:
     def __init__(self):
         self.counter = 0
@@ -169,49 +189,58 @@ nestedform = widgets.TableForm(fields=[
             widgets.TextField("age")
         ])
     ])
+
+
 class NestedController(controllers.Controller):
-    def checkform(self, foo):
-        self.foo = foo
-    checkform = expose()(checkform)
-    checkform = validate(form=nestedform)(checkform)
+
+    @expose()
+    @validate(form=nestedform)
+    def checkform(self, foo=None):
+    #def checkform(self, foo=None, *args, **kw):
+        #raise Exception('HolyHell: %s: %s. kw: %s' % (foo, args, kw))
+        return foo
+
+    @expose()
+    def field_for(self):
+        cherrypy.request.validation_errors = dict(foo=dict(foo='error'))
+        template = """\
+        <div xmlns:py="http://purl.org/kid/ns#">
+            ${field_for('foo').fq_name}.appears
+            ${field_for('foo').error}_appears
+            ${field_for('foo').field_id}_appears
+            ${field_for('foo').display(value_for('foo'), **params_for('foo'))}
+        </div>
+        """
+        textfield = widgets.TextField("foo")
+        fieldset = widgets.FieldSet("foo", fields=[textfield], template=template)
+        form = widgets.Form("form", fields=[fieldset], template=template)
+
+        # Good example below of how you can pass parameters and values to nested
+        # widgets.
+        value = dict(foo=dict(foo="HiYo!"))
+        params = dict(attrs=dict(foo=dict(foo=dict(size=100))))
+        params['format'] = 'xhtml'
+        return form.render(value, **params)
+
 
 def test_nested_variables():
-    cherrypy.request = oldrequest
-    newroot = NestedController()
-    cherrypy.root = None
-    cherrypy.tree.mount_points = {}
-    cherrypy.tree.mount(newroot, "/")
     url = u"/checkform?foo.name=Kevin&foo.age=some%20Numero".encode("utf-8")
-    create_request(url)
-    assert config.get("decoding_filter.encoding", path="/") == "utf8"
-    assert newroot.foo
-    assert newroot.foo["name"] == "Kevin"
-    assert newroot.foo["age"] == u"some Numero"
+    testutil.stop_server(tg_only = True)
+    app = testutil.make_app(NestedController)
+    testutil.start_server()
+    request = app.get(url)
+    assert config.get("decoding_filter.encoding", path='/') == "utf8"
+    assert request.raw["name"] == "Kevin"
+    assert request.raw["age"] == u"some Numero"
 
 def test_field_for():
-    cherrypy.request.validation_errors = dict(foo=dict(foo='error'))
-    template = """\
-    <div xmlns:py="http://purl.org/kid/ns#">
-        ${field_for('foo').fq_name}.appears
-        ${field_for('foo').error}_appears
-        ${field_for('foo').field_id}_appears
-        ${field_for('foo').display(value_for('foo'), **params_for('foo'))}
-    </div>
-    """
-    textfield = widgets.TextField("foo")
-    fieldset = widgets.FieldSet("foo", fields=[textfield], template=template)
-    form = widgets.Form("form", fields=[fieldset], template=template)
-    # Good example below of how you can pass parameters and values to nested
-    # widgets.
-    value = dict(foo=dict(foo="HiYo!"))
-    params = dict(attrs=dict(foo=dict(foo=dict(size=100))))
-    params['format'] = 'xhtml'
-    output = form.render(value, **params)
-    assert "form_foo_appears" in output
-    assert "form_foo_foo_appears" in output
-    assert "foo.appears" in output
-    assert "foo.foo.appears" in output
-    assert "error_appears" in output
-    assert "textfield" in output
-    assert "HiYo!" in output
-    assert "size=\"100\"" in output
+    app = make_app(NestedController)
+    response = app.get('/field_for')
+    assert "form_foo_appears" in response
+    assert "form_foo_foo_appears" in response
+    assert "foo.appears" in response
+    assert "foo.foo.appears" in response
+    assert "error_appears" in response
+    assert "textfield" in response
+    assert "HiYo!" in response
+    assert "size=\"100\"" in response
